package com.flyfish.framework.compiler.core;

import org.apache.commons.lang3.StringUtils;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Load class from byte[] which is compiled in memory.
 *
 * @author michael
 */
class MemoryClassLoader extends URLClassLoader {

    // class name to class bytes:
    private final Map<String, byte[]> classBytes = new ConcurrentHashMap<>();

    public MemoryClassLoader() {
        super(new URL[0], MemoryClassLoader.class.getClassLoader());
    }

    @Override
    protected Class<?> findClass(String name) throws ClassNotFoundException {
        byte[] buf = classBytes.get(name);
        if (buf == null) {
            return super.findClass(name);
        }
        classBytes.remove(name);
        return defineClass(name, buf);
    }

    public Map<String, byte[]> getClassBytes() {
        return classBytes;
    }

    /**
     * 修复获取package为null的问题，定义并验证class
     *
     * @param name 名称
     * @param buf  二进制数据
     * @return 结果
     */
    @SuppressWarnings("all")
    private Class<?> defineClass(String name, byte[] buf) {
        // 先定义class，然后在上下文基础上判断package有效性
        Class<?> defined = defineClass(name, buf, 0, buf.length);
        synchronized (defined) {
            // 补偿定义包，防止缺失
            if (null == defined.getPackage()) {
                // 包名
                String packageName = StringUtils.substringBeforeLast(name, ".");
                definePackage(packageName, null, null,
                        null, null, null, null, null);
            }
        }
        return defined;
    }
}
