package com.flyfish.framework.utils;


import com.flyfish.framework.enums.CodeBasedEnum;

import java.util.NoSuchElementException;

/**
 * 枚举工具类
 *
 * @author Liu.xT
 */
public class EnumUtils {
    /**
     * 根据枚举标识码获取枚举对象
     *
     * @param code      枚举标识
     * @param enumClass 枚举Class类型
     * @param <E>       枚举泛型
     * @return 枚举对象
     * @throws NoSuchElementException 找不到指定元素
     */
    public static <E extends Enum<?> & CodeBasedEnum> E getByCode(int code, Class<E> enumClass) throws NoSuchElementException {
        E[] enumConstants = enumClass.getEnumConstants();
        for (E e : enumConstants) {
            if (e.getCode() == code) {
                return e;
            }
        }
        throw new NoSuchElementException("找不到标识码[" + code + "]对应的枚举对象: [" + enumClass.getSimpleName() + "]");
    }

}
