package com.flyfish.framework.controller;

import com.flyfish.framework.bean.EnumValue;
import com.flyfish.framework.bean.Result;
import com.flyfish.framework.enums.NamedEnum;
import com.flyfish.framework.utils.StringFormats;
import org.apache.commons.lang3.ClassUtils;
import org.reflections.Reflections;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 统一处理枚举
 * 将来会加入字典自动处理
 *
 * @author wangyu
 */
@RestController
@RequestMapping("enums/{code}")
public class EnumController {

    // 映射管理，避免class.forName扫描包路径
    private final Map<String, Map<String, String>> mapper = new HashMap<>();

    private final Map<String, List<EnumValue>> values = new HashMap<>();

    public EnumController() {
        Reflections reflections = new Reflections("com.flyfish.project", "com.flyfish.framework");
        // 得到Resource注解的类
        Set<Class<? extends NamedEnum>> classSet = reflections.getSubTypesOf(NamedEnum.class);
        // 注入
        classSet.stream().filter(clazz -> ClassUtils.isAssignable(clazz, Enum.class)).forEach(clazz -> {
            String name = StringFormats.camel2Line(ClassUtils.getShortClassName(clazz));
            List<EnumValue> values = Arrays.stream(clazz.getEnumConstants()).reduce(new ArrayList<>(), (result, item) -> {
                result.add(new EnumValue(((Enum<?>) item).name(), item.getName()));
                return result;
            }, (a, b) -> a);
            this.mapper.put(name, values.stream().collect(Collectors.toMap(EnumValue::getValue, EnumValue::getText)));
            this.values.put(name, values);
        });
    }

    /**
     * 通过code和key查找显示名称
     *
     * @param code 编码
     * @param key  键
     * @return 结果
     */
    @GetMapping("{key}")
    public Result<String> getText(@PathVariable("code") String code, @PathVariable("key") String key) {
        return Result.ok(mapper.getOrDefault(code, Collections.emptyMap()).get(key));
    }

    /**
     * 值的集合
     *
     * @param code 编码
     * @return 结果
     */
    @GetMapping
    public Result<?> values(@PathVariable("code") String code) {
        if ("all".equals(code)) {
            return Result.ok(values);
        }
        return Result.ok(values.getOrDefault(code, Collections.emptyList()));
    }
}
