package com.flyfish.framework.transform;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.utils.JacksonUtil;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferFactory;
import reactor.core.publisher.Mono;

import java.nio.charset.Charset;

/**
 * Json返回值转换器
 *
 * @author wangyu
 */
public class ResultDataTransformer implements DataBufferTransformer<Result<?>> {

    private final DataBufferFactory factory;

    public ResultDataTransformer() {
        this.factory = SharedDataBufferFactory.getDefaultFactory();
    }

    @Override
    public Mono<DataBuffer> transform(Result<?> source) {
        return JacksonUtil.toJson(source)
                .map(result -> result.getBytes(Charset.defaultCharset()))
                .map(factory::wrap)
                .map(Mono::just)
                .orElse(Mono.empty());
    }
}
